{-
Copyright 2009 Christian Sternagel, René Thiemann, Sarah Winkler, Harald Zankl

This file is part of IsaFoR/CeTA.

IsaFoR/CeTA is free software: you can redistribute it and/or modify it under the
terms of the GNU Lesser General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

IsaFoR/CeTA is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along
with IsaFoR/CeTA. If not, see <http://www.gnu.org/licenses/>.
-}
module ProofChecker where {

import DepGraph;
import List;
import Nat;
import ErrorMonad;
import Polynomial;
import Trs;
import Show;
import Term;
import Util;
import DpTransImpl;

data DPProof a b = DepGraphProc [(Maybe (DPProof a b), [(Term a b, Term a b)])]
  | RedPairProc (RedPair a) [(Term a b, Term a b)] (DPProof a b) | PisEmpty
  deriving (Read, Show);

indent :: [[Char]] -> [[Char]];
indent xs = "\n" : mapa (append "   ") xs;

data TRSProof a b =
  DPTrans [(Term (Shp a) b, Term (Shp a) b)] (DPProof (Shp a) b)
  deriving (Read, Show);

checkDPProof ::
  forall a b.
    (Eq a, Sshow a, Eq b,
      Sshow b) => [(Term a b, Term a b)] ->
                    [(Term a b, Term a b)] ->
                      DPProof a b -> Eithera [[Char]] ();
checkDPProof p r (DepGraphProc edpts) =
  bind (catcha
         (checkDepGraphProc p
           (mapa (\ x ->
                   ((case fst x of {
                      Nothing -> False;
                      Just xa -> True;
                    }),
                     snd x))
             edpts)
           r)
         (\ x ->
           errora
             ("error while trying to perform SCC-decomposition  on " :
               append (showDPP p r) x)))
    (\ x ->
      catcha
        (catcha
          (checkAllMI
            (\ rp ->
              (case fst rp of {
                Nothing -> returna ();
                Just a -> checkDPProof (snd rp) r a;
              }))
            edpts)
          (\ xa -> errora (snd xa)))
        (\ xa ->
          errora ("error below the dependency graph processor" : indent xa)));
checkDPProof p r (RedPairProc redp rP prf) =
  bind (catcha (createRedPairProc redp p rP r)
         (\ x ->
           errora
             (append (showDPP p r)
               ("error when applying the reduction pair processor to obtain" :
                 append (showP rP) x))))
    (\ x ->
      catcha (checkDPProof rP r prf)
        (\ xa ->
          errora ("error below the reduction pair processor" : indent xa)));
checkDPProof p r PisEmpty =
  (case p of {
    [] -> returna ();
    x : xa ->
      errora ("P is not empty in the following DP-problem " : showDPP p r);
  });

checkTRSProof ::
  forall a b.
    (Eq a, Sshow a, Eq b,
      Sshow b) => [(Term a b, Term a b)] -> TRSProof a b -> Eithera [[Char]] ();
checkTRSProof r (DPTrans p prf) =
  bind (catcha (checkDPs r p)
         (\ x ->
           errora
             ("error when switching from the TRS" :
               append (showTRS r)
                 ("to the initial DP problem" :
                   append (showDPP p (computeID r)) x))))
    (\ x ->
      catcha (checkDPProof p (computeID r) prf)
        (\ xa ->
          errora ("error below switch to dependency pairs" : indent xa)));

}
